
#include <s11n.net/s11n/io/sqlite3/s11nlite_api.hpp>
#include <s11n.net/s11n/io/sqlite3/sqlite3_serializer.hpp>

/************************************************************************
ACHTUNG:

Linking this file against your app will cause sqlite3_serializer to take
over the s11nlite API. Don't do this unless you really want to.

************************************************************************/
namespace s11n { namespace io { namespace sqlite3 {

	s11nlite_api::s11nlite_api() {}
	s11nlite_api::~s11nlite_api(){}


	bool s11nlite_api::save( const s11nlite_api::node_type & src, std::ostream & dest )
	{

//		CERR << "s11nlite_api::save(node,ostream)\n";
		sqlite3_serializer<s11nlite::node_type> ser;
		return ser.serialize( src, dest );
	}
	s11nlite_api::node_type * s11nlite_api::load_node( std::istream & src )
	{
//		CERR << "s11nlite_api::load(istream)\n";
		sqlite3_serializer<s11nlite::node_type> ser;
		return ser.deserialize( src );
	}

	bool s11nlite_api::save( const s11nlite_api::node_type & src, const std::string & dest )
	{
		// CERR << "s11nlite_api::save(node,string["<<dest<<"])\n";
		sqlite3_serializer<s11nlite::node_type> ser;
		return ser.serialize( src, dest );
	}

	s11nlite_api::node_type * s11nlite_api::load_node( const std::string & src )
	{
		// CERR << "s11nlite_api::load(string["<<src<<"])\n";
		/**
			Maintenance/design notes:

			We open the file before continuing so we can see if it's
			a db file or not.

			All this work really isn't necessary: s11n
			1.2.1 was patched up to more transparently
			support loading of DBs.
		*/
		std::string cookie( s11n::io::get_magic_cookie( src ) );
		node_type * node = 0;
		// CERR << "cookie == [" << cookie <<"]\n";
		// if( s11n_SQLITE_MAGIC_COOKIE == cookie ) // weird: compares false???
		if( std::string::npos != cookie.find( s11n_SQLITE_MAGIC_COOKIE ) )
		{
			// CERR << "s11nlite_api::load(string["<<src<<"]) DB LOAD\n";
			sqlite3_serializer<s11nlite::node_type> ser;
			node = ser.deserialize( src );
		}
		else
		{
			// CERR << "s11nlite_api::load(string["<<src<<"]) NORMAL LOAD\n";
			node = this->s11nlite::client_interface::load_node( src );
		}
		return node;
	}

}}} // namespace s11n::io::sqlite3
