#include <string>
#include <iostream>
#include <sstream>
#include <unistd.h> // unlink()

#include <s11n.net/s11n/io/sqlite3/sqlite3_util.hpp>

#ifndef CERR
#  define CERR std::cerr << __FILE__<<":"<<__LINE__<<" "
#endif

#define SQERR_PREFIX "****** "
#define SQERR std::cerr << SQERR_PREFIX << "SQL Error: "


int istream_to_db( std::istream & is, ::sqlite3 * db )
{
	int rc = 0;
	std::string line;
	std::string buff;
	std::string errmsg;
	buff = "";
	size_t count =0;
	while( std::getline( is, line ) )
	{
		++count;
		if( line.empty() ) continue;
		buff += line;
		if( sqlite3_complete( buff.c_str() ) )
		{
			rc = sqlite3_exec( db, buff.c_str(), 0, 0, 0 );
			if( 0 != rc )
			{
				SQERR << "near line #"<<count<<": " << sqlite3_errmsg( db ) << '\n'
				      << SQERR_PREFIX << "SQL CODE=" << line << std::endl;
			}
			buff = "";
			//if( 0 != rc ) break;
		}
		else
		{
			buff += '\n';
		}
	}
	if( ! buff.empty() )
	{
		rc = sqlite3_exec( db, buff.c_str(), 0, 0, 0 );
		if( 0 != rc )
		{
			SQERR << "near line #"<<count<<": " << sqlite3_errmsg( db ) << '\n'
			      << SQERR_PREFIX << "SQL CODE=" << buff << std::endl;
		}
	}

	return rc;
}

int stream_db( std::istream & is, std::string const & dbfile )
{
	::s11n::io::sqlite3::DBScope dbw( dbfile );
	return istream_to_db( is, dbw.db );
}

void show_help( std::string const & appname )
{
	std::cout << appname << " - a utility to feed SQL to sqlite3 databases.\n"
		  << "Usage: " << appname << " database\n"
		  << "It reads SQL code from stdin until EOF, passing it all to the database.\n"
		  << "This utility is part of the sqlite3_serializer package.\n"
		  << std::endl;
		;
}

int main( int argc, char **argv )
{
	std::string arg;
	for( int i = 1; i < argc; ++i )
	{
		arg = argv[i];
		if( ("-?" == arg) || ("-help" == arg)
		    || ("--help" == arg)
		    )
		{
			show_help(argv[0]);
			return 0;
		}
	}

	try
	{

		if( 2 == argc )
		{
			return stream_db( std::cin, argv[1] );
		}
		show_help(argv[0]);
		return 1;

	}
	catch( const std::exception & ex )
	{
		CERR << argv[0] << ": exception: " << ex.what() << std::endl;
		return 10;
	}
	return 0;
}

