#include <string>
#include <iostream>
#include <sstream>
#include <unistd.h> // unlink()


#include <s11n.net/s11n/s11nlite.hpp>
#include <s11n.net/s11n/io/sqlite3/sqlite3_util.hpp>
#include <s11n.net/s11n/io/sqlite3/sqlite3_serializer.hpp>
#include <s11n.net/s11n/io/sqlite3/s11nlite_api.hpp>

#ifndef CERR
#  define CERR std::cerr << __FILE__<<":"<<__LINE__<<" "
#endif


// int sql_result_callback(void *NotUsed, int argc, char **argv, char **azColName){
// 	if( 0 != NotUsed )
// 	{
// 		int i;
// 		for(i=0; i<argc; i++){
// 			printf("%s = %s\n", azColName[i], argv[i] ? argv[i] : "NULL");
// 		}
// 		printf("\n");
// 	}
// 	return 0;
// }

template <typename NodeType>
bool convert_to_db( NodeType const & src, ::sqlite3 * db )
{
	namespace sq = ::s11n::io::sqlite3;
	return 0 != sq::save_node<s11nlite::node_type>( src, db );
}



bool convert_to_db( std::string const & infile, std::string const & outfile )
{
	std::auto_ptr<s11nlite::node_type> node( s11nlite::load_node( infile ) );
	if( ! node.get() )
	{
		throw std::runtime_error( "Error loading s11nnode from "+infile );
	}

	int rc = 0;
	namespace sq = ::s11n::io::sqlite3;
	sq::DBScope dbw(outfile);
	::sqlite3 *db = dbw.db;
	try
	{
		rc = sq::save_node<s11nlite::node_type>( *node, db ) ? 0 : 1;
#if 0 // some debugging code
		std::auto_ptr<s11nlite::node_type> renode( sq::load_node<s11nlite::node_type>( db ) );
		if( renode.get() )
		{
			CERR << "Re-ported from db to an s11n_node tree:\n";
			// s11nlite::save( *renode, outfile+".second" );
			s11nlite::save( *renode, std::cerr );
		}
#endif // some debugging code
	}
	catch( ... )
	{
		throw;
	}
	// CERR << "Cookie of output file="<<s11n::io::get_magic_cookie( outfile ) << '\n';

	return 0 == rc;
}

int convert_to_stdout( std::string const & file )
{
	std::auto_ptr<s11nlite::node_type> node( s11nlite::load_node( file ) );
	if( ! node.get() )
	{
		throw s11n::s11n_exception( "Could not de-s11n file: "+file,__FILE__,__LINE__ );
	}

	// s11n::io::sqlite3::save_node( *node, std::cout );
	s11nlite::save( *node, std::cout );

	return 0;
}

void bogo_tests()
{
	namespace sq = ::s11n::io::sqlite3;
	std::string dbfile("test.db");
	unlink(dbfile.c_str());
	sq::DBScope dbw(dbfile);

	std::ostringstream sql;
	sql << "create table t(a,b);";
	dbw.exec( sql.str(), true );
	sql.str("");
	for( int i = 0; i < 10; ++i )
	{
		sql << "insert into t values("<<i<<","<<i+1<<");\n";
	}
	dbw.exec( sql.str() );
	sql.str("");
	sql << "select * from t order by b desc;";
	::sqlite3_stmt * pst = 0;
	char const * tail = 0;
	int rc = sqlite3_prepare( dbw.db,
				  sql.str().c_str(),
				  sql.str().size(),
				  &pst,
				  &tail );
	if( rc ) throw std::runtime_error( "rc != 0" );
	if( ! pst ) throw std::runtime_error( "pst == 0" );
	const int cols = sqlite3_column_count(pst);
	int i =0 ;
	for( ; SQLITE_ROW == (rc = sqlite3_step(pst)); ++i )
	{
		for( int c = 0; c < cols; ++c )
		{
			CERR << "Result #"<<i <<":\t"
			     << sqlite3_column_name(pst,c) << "="<<
				sqlite3_column_text(pst,c) << "\n";
		}
	}
	sqlite3_finalize( pst );
	CERR << i << " row(s). RC=="<<rc<<'\n';

}

/** An sqlite experiment. */
class DBExec
{
public:
	typedef std::vector<std::string> string_list;
	DBExec( ::sqlite3 * db, std::string const & sql, bool throwOnError = false );

	int result_code() const { return this->m_rc; }

	std::string col_name( int i );
	string_list col_names();
	std::string row_value( int i );
	string_list row_values();

	bool has_row();
	bool next_row();

	int row_count();
	int column_count();

private:
	int m_rc;
	::sqlite3 * m_db;
	std::string m_sql;
};

int test_table()
{
	return 0;
}

int main( int argc, char **argv )
{
	namespace sq = ::s11n::io::sqlite3;

	// bogo_tests();


	// linking against s11nlite_sqlite3.so does the same as:
	//sq::s11nlite_api liteapi;
	//s11nlite::instance( &liteapi );


	// ^^^ Now all FILE-based s11nlite::load()/save() calls will be routed
	// through sqlite3.
	try
	{
		// bogo_tests(); return 0;

		if( 2 == argc )
		{
			return convert_to_stdout( argv[1] );
		}
		else if( 3 == argc )
		{
			return convert_to_db( argv[1], argv[2] ) ? 0 : 1;
		}

		CERR << "Try one of:\n"
		     << argv[0] << " input_file\n"
		     << argv[0] << " input_s11n_or_db_file output_db_file\n"
			;
		return 1;

	}
	catch( const std::exception & ex )
	{
		std::cerr << argv[0] << ": exception: " << ex.what() << std::endl;
		return 10;
	}
	return 0;
}

