#include <s11n.net/s11n/io/sqlite3/wrapper.hpp>
#include <sstream>

namespace sq3 {

	db_error::db_error( const std::string & What ) : m_what(What)
	{
	}

	const char * db_error::what() const throw()
	{
		return this->m_what.c_str();
	}

	db_error::db_error( ::sqlite3 * db )
	{
		char const * e = db ? sqlite3_errmsg( db ) : 0;
		this->m_what = e ? e : "";
	}

        db_error::db_error( const std::string & what,
			    const std::string & file,
			    unsigned int line )
        {
                std::ostringstream os;
                os << file << ":"<<line<<": " << what;
                this->m_what = os.str();
        }


	database::database( std::string const & dbfile ) : db(0)
	{
        	int rc = sqlite3_open(dbfile.c_str(), &this->m_db);
	        if( rc )
        	{
			this->close();
	                throw db_error( std::string("Can't open database file "+dbfile+": ")+this->errmsg(this->db) );
        	}
	}

	int database::exec( std::string const & sql, bool throwOnError )
	{
		int rc = this->m_db
			? sqlite3_exec( this->db, sql.c_str(), 0,0,0 )
			: -1;
		if( (0==rc) || ((0!=rc) && !throwOnError) ) return rc;
		throw db_error( std::string("database::exec(): SQL error: ")+this->errmsg()+"\nSQL="+sql  );
		return 0; // avoid warning from over-zealous compilers
	}

	void database::open( std::string const & sql )
	{
		int rc = this->db
			? sqlite3_exec( this->db, sql.c_str(), 0,0,0 )
			: -1;
		if( !rc ) return rc;
		throw db_error( std::string("database::exec(): SQL error: ")+this->errmsg()+"\nSQL="+sql  );
		return 0; // avoid warning from over-zealous compilers
	}

	std::string database::errmsg()
	{
		return sqlite3_errmsg(this->db() );
	}

	void database::close()
	{
		if( this->db )
		{
			sqlite3_close( this->db() );
			this->m_db = 0;
		}
	}

	database::~database()
	{
		this->close();
	}

}
