#ifndef sq3_WRAPPER_HPP_INCLUDED
#define sq3_WRAPPER_HPP_INCLUDED 1

#include <sqlite3.h>
#include <string>

namespace sq3 {

	struct db_error : public std::exception
        {
	public:
		virtual ~db_error() throw() {}
                explicit db_error( const std::string & What );

		/**
		   Creates a what() string of sqlite3_errmsg(db).
		*/
                explicit db_error( ::sqlite3 * db );
		/**
		   Creates an error string consisting of:

		   file:linenum: What
		*/
                db_error( const std::string & What, const std::string & file, unsigned int linenum );
                virtual const char * what() const throw();
        private:
                std::string m_what;
	};

	class database
	{
	public:
		/**
		   open()s infile. Throws a std::exception on error.
		*/
		database( std::string const & infile );

		/**
		   Creates an object which must be open()ed before it
		   can be used.
		*/
		database();

		/**
		   Calls this->close().
		*/
		~database();

		/**
		   DB handle. This object owns it.
		*/
		::sqlite3 * db() { return this->m_db; }

		std::string errmsg();

		/**
		   Returns sqlite3_exec( this->db, sql.c_str(), 0,0,0 ),
		   or -1 if (! this->db).

		   If throwOnError is true then a non-0 return code will cause
		   an exception to be thrown.
		*/
		int exec( std::string const & sql, bool throwOnError = true );

		/** Opens the given db. Throws on error. */
		void open( std::string const & filename );

		/**
		   Calls s1lite3_close() on this->db(). It should
		   never be necessary to manually call this unless you
		   want to clean up your db connection long before
		   this object goes out of scope.
		*/
		void close();

	private:
		::sqlite3 * m_db;
		database( database const & ); // Unimplemented. Noncopyable.
		database & operator=( database const & ); // Unimplemented. Noncopyable.
	};


} // namespace sq3

#endif // sq3_WRAPPER_HPP_INCLUDED
